@@ -311,8 +311,8 @@ class TestMRCommands(TestCommands):
         expected = [{'value': {'min': docs[0]['timestamp'],
                                'max': docs[-1]['timestamp']},
                      '_id': 'timestamp'}]
-        print 'RESULTS:', result['results']
-        print 'EXPECTED:', expected
+        print('RESULTS:', result['results'])
+        print('EXPECTED:', expected)
         self.assertEqual(result['results'], expected)
 
     def test_mr_inline_collection(self):
@@ -615,7 +615,8 @@ class TestCollection(TestCase):
     def test_insert_manipulate_true(self):
         doc = {'x': 1}
         sample_id = bson.ObjectId()
-        with patch('bson.ObjectId', return_value=sample_id):
+        # Cannot patch the class itself, otherwise isinstance() checks will fail on PyPy
+        with patch('bson.ObjectId.__init__', autospec=True, return_value=None, side_effect=lambda *args: args[0]._ObjectId__validate(sample_id)):
             self.bind.db.coll.insert(doc, manipulate=True)
         self.assertEqual(doc, {'x': 1, '_id': sample_id})
 
@@ -638,6 +639,12 @@ class TestBsonCompare(TestCase):
         assert mim.BsonArith.cmp(False, bson.ObjectId()) == 1
         assert mim.BsonArith.cmp(True, datetime.fromordinal(1)) == -1
 
+    def test_float_bson_type(self):
+        assert mim.BsonArith.cmp(1, 1) == 0
+        assert mim.BsonArith.cmp(1.1, 1.1) == 0
+        assert mim.BsonArith.cmp(1.1, -1.3) == 1
+        assert mim.BsonArith.cmp(1.1, 1.1111) == -1
+
 class TestMatch(TestCase):
 
     def test_simple_match(self):
