@@ -1,27 +1,9 @@
 import cgi
-import urllib
+import six
+from six.moves import urllib
 from threading import local
 
-def parse_uri(uri, **kwargs):
-    scheme, rest = urllib.splittype(uri)
-    host, rest = urllib.splithost(rest)
-    user, host = urllib.splituser(host)
-    if user:
-        username, password = urllib.splitpasswd(user)
-    else:
-        username = password = None
-    host, port = urllib.splitnport(host)
-    path, query = urllib.splitquery(rest)
-    if query:
-        kwargs.update(dict(cgi.parse_qsl(query)))
-    return dict(
-        scheme=scheme,
-        host=host,
-        username=username,
-        password=password,
-        port=port,
-        path=path,
-        query=kwargs)
+import pymongo
 
 class EmptyClass(object): pass
 
@@ -37,6 +19,37 @@ class LazyProperty(object):
         result = obj.__dict__[self.__name__] = self._func(obj)
         return result
 
+class ContextualProxy(object):
+
+    def __init__(self, cls, context, *args, **kwargs):
+        self._cls = cls
+        self._context = context
+        self._args = args
+        self._kwargs = kwargs
+        self._registry = {}
+
+    def _get(self):
+        ctx = self._context()
+        try:
+            return self._registry[ctx]
+        except KeyError:
+            result = self._cls(*self._args, **self._kwargs)
+            self._registry[ctx] = result
+            return result
+
+    def __getattr__(self, name):
+        return getattr(self._get(), name)
+
+    def __repr__(self):
+        return 'CProxy of %r' % self._get()
+
+    def close(self):
+        ctx = self._context()
+        try:
+            del self._registry[ctx]
+        except AttributeError:
+            pass
+
 class ThreadLocalProxy(object):
 
     def __init__(self, cls, *args, **kwargs):
@@ -103,3 +116,23 @@ def wordwrap(s, width=80, indent_first=0, indent_subsequent=0):
 def indent(s, level=2):
     prefix = ' ' * level
     return s.replace('\n', '\n' + prefix)
+
+def fixup_index(index, direction=pymongo.ASCENDING):
+
+    def _fixup(i):
+        if isinstance(i, six.string_types):
+            yield (i, direction)
+        elif (isinstance(i, tuple)
+              and len(i) == 2
+              and i[1] in (pymongo.ASCENDING, pymongo.DESCENDING)):
+            yield i
+        elif (isinstance(i, tuple)
+             and len(i) == 2
+             and i[1] == pymongo.GEO2D):
+            yield i
+        else:
+            for key in i:
+                for x in _fixup(key): yield x
+
+    return list(_fixup(index))
+
